@@ -862,7 +862,7 @@ static SysVar SysVarArr[] = {
     {"DefaultColor",   1,  SPECIAL_TYPE, default_color_func,   0,      0 },
     {"DefaultDelta",   1,  INT_TYPE,     &DefaultDelta,        0,      10000 },
     {"DefaultPrio",    1,  INT_TYPE,     &DefaultPrio,         0,      9999 },
-    {"DefaultTDelta",  1,  INT_TYPE,     &DefaultTDelta,       0,      1440 },
+    {"DefaultTDelta",  1,  INT_TYPE,     &DefaultTDelta,       0,      MINUTES_PER_DAY },
     {"DeltaOverride",  0,  INT_TYPE,     &DeltaOverride,       0,      0 },
     {"DontFork",       0,  INT_TYPE,     &DontFork,            0,      0 },
     {"DontQueue",      0,  INT_TYPE,     &DontQueue,           0,      0 },
@@ -898,7 +898,7 @@ static SysVar SysVarArr[] = {
     {"LongSec",        1,  SPECIAL_TYPE, longsec_func,         0,      0 },
     {"March",          1,  TRANS_TYPE,   "March",              0,      0 },
     {"MaxFullOmits",   0,  CONST_INT_TYPE, NULL,        MAX_FULL_OMITS, 0},
-    {"MaxLateMinutes", 1,  INT_TYPE,     &MaxLateMinutes,      0,      1440 },
+    {"MaxLateMinutes", 1,  INT_TYPE,     &MaxLateMinutes,      0,      MINUTES_PER_DAY },
     {"MaxPartialOmits",0,  CONST_INT_TYPE, NULL,    MAX_PARTIAL_OMITS, 0},
     {"MaxSatIter",     1,  INT_TYPE,     &MaxSatIter,          10,     ANY },
     {"MaxStringLen",   1,  INT_TYPE,     &MaxStringLen,        -1,     ANY },
